# -*- coding: utf-8 -*-

# Copyright 2015-2019 Mike Fährmann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extract manga-chapters from https://dynasty-scans.com/"""

from .common import ChapterExtractor, Extractor, Message
from .. import text
import json
import re


BASE_PATTERN = r"(?:https?://)?(?:www\.)?dynasty-scans\.com"


class DynastyscansBase():
    """Base class for dynastyscans extractors"""
    category = "dynastyscans"
    root = "https://dynasty-scans.com"

    def _parse_image_page(self, image_id):
        url = "{}/images/{}".format(self.root, image_id)
        page = self.request(url).text

        date, pos = text.extract(page, "class='create_at'>", "</span>")
        tags, pos = text.extract(page, "class='tags'>", "</span>", pos)
        src , pos = text.extract(page, "class='btn-group'>", "</div>", pos)
        url , pos = text.extract(page, ' src="', '"', pos)

        src = text.extract(src, 'href="', '"')[0] if "Source<" in src else ""

        return {
            "url": self.root + url,
            "image_id": text.parse_int(image_id),
            "tags": text.split_html(text.unescape(tags)),
            "date": text.remove_html(date),
            "source": text.unescape(src),
        }


class DynastyscansChapterExtractor(DynastyscansBase, ChapterExtractor):
    """Extractor for manga-chapters from dynasty-scans.com"""
    pattern = BASE_PATTERN + r"(/chapters/[^/?&#]+)"
    test = (
        (("http://dynasty-scans.com/chapters/"
          "hitoribocchi_no_oo_seikatsu_ch33"), {
            "url": "dce64e8c504118f1ab4135c00245ea12413896cb",
            "keyword": "1564965671ac69bb7fbc340538397f6bd0aa269b",
        }),
        (("http://dynasty-scans.com/chapters/"
          "new_game_the_spinoff_special_13"), {
            "url": "dbe5bbb74da2edcfb1832895a484e2a40bc8b538",
            "keyword": "22b35029bc65d6d95db2e2c147b0a37f2d290f29",
        }),
    )

    def metadata(self, page):
        info  , pos = text.extract(page, "<h3 id='chapter-title'><b>", "</b>")
        author, pos = text.extract(page, " by ", "</a>", pos)
        group , pos = text.extract(page, '"icon-print"></i> ', '</span>', pos)
        date  , pos = text.extract(page, '"icon-calendar"></i> ', '<', pos)

        match = re.match(
            (r"(?:<a[^>]*>)?([^<]+)(?:</a>)?"  # manga name
             r"(?: ch(\d+)([^:<]*))?"  # chapter info
             r"(?:: (.+))?"),  # title
            info
        )

        return {
            "manga": text.unescape(match.group(1)),
            "chapter": text.parse_int(match.group(2)),
            "chapter_minor": match.group(3) or "",
            "title": text.unescape(match.group(4) or ""),
            "author": text.remove_html(author),
            "group": (text.remove_html(group) or
                      text.extract(group, ' alt="', '"')[0] or ""),
            "date": date,
            "lang": "en",
            "language": "English",
        }

    def images(self, page):
        data = text.extract(page, "var pages = ", ";\n")[0]
        return [
            (self.root + img["image"], None)
            for img in json.loads(data)
        ]


class DynastyscansSearchExtractor(DynastyscansBase, Extractor):
    """Extrator for image search results on dynasty-scans.com"""
    subcategory = "search"
    directory_fmt = ("{category}", "Images")
    filename_fmt = "{image_id}.{extension}"
    archive_fmt = "i_{image_id}"
    pattern = BASE_PATTERN + r"/images(?:\?([^#]+))?$"
    test = ("https://dynasty-scans.com/images?with[]=4930&with[]=5211", {
        "url": "6b570eedd8a741c2cd34fb98b22a49d772f84191",
        "keyword": "2a8f3d30584c637a0dd64ce8a0a2e81edaa6bca4",
    })

    def __init__(self, match):
        Extractor.__init__(self, match)
        self.search_query = match.group(1)

    def items(self):
        yield Message.Version, 1
        yield Message.Directory, {}
        for image_id in self.images():
            data = self._parse_image_page(image_id)
            url = data.pop("url")
            yield Message.Url, url, text.nameext_from_url(url, data)

    def images(self):
        url = self.root + "/images?" + self.search_query
        params = {"page": 1}

        while True:
            page = self.request(url, params=params).text
            yield from text.extract_iter(page, '"/images/', '"')
            if 'rel="next"' not in page:
                return
            params["page"] += 1


class DynastyscansImageExtractor(DynastyscansBase, Extractor):
    """Extractor for individual images on dynasty-scans.com"""
    subcategory = "image"
    directory_fmt = ("{category}", "Images")
    filename_fmt = "{image_id}.{extension}"
    pattern = BASE_PATTERN + r"/images/(\d+)"
    test = ("https://dynasty-scans.com/images/1245", {
        "url": "15e54bd94148a07ed037f387d046c27befa043b2",
        "keyword": "384889567a19d2e907ff13f65b42f9560e15172d",
    })

    def __init__(self, match):
        Extractor.__init__(self, match)
        self.image_id = match.group(1)

    def items(self):
        data = self._parse_image_page(self.image_id)
        url = data.pop("url")

        yield Message.Version, 1
        yield Message.Directory, data
        yield Message.Url, url, text.nameext_from_url(url, data)



# -*- coding: utf-8 -*-

# Copyright 2019 Mike Fährmann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extractors for http://blog.livedoor.jp/"""

from .common import Extractor, Message
from .. import text


class LivedoorExtractor(Extractor):
    """Base class for livedoor extractors"""
    category = "livedoor"
    root = "http://blog.livedoor.jp"
    img_root = "http://livedoor.blogimg.jp"
    filename_fmt = "{post[id]}_{post[title]}_{num:>02}.{extension}"
    directory_fmt = ("{category}", "{post[user]}")
    archive_fmt = "{post[id]}_{hash}"

    def __init__(self, match):
        Extractor.__init__(self, match)
        self.user = match.group(1)

    def items(self):
        yield Message.Version, 1
        for post in self.posts():
            images = self._images(post)
            if images:
                yield Message.Directory, {"post": post}
                for image in images:
                    yield Message.Url, image["url"], image

    def posts(self):
        """Return an iterable with post objects"""

    def _load(self, data, body):
        pid  , pos = text.extract(data, "id : '"   , "'")
        title, pos = text.extract(data, "title : '", "'", pos)
        cat1 , pos = text.extract(data, "name:'"   , "'", pos)
        cat2 , pos = text.extract(data, "name:'"   , "'", pos)
        date , pos = text.extract(data, "date : '" , "'", pos)
        tags , pos = text.extract(body, '</dt><dd>', '</dl>')

        return {
            "id"        : text.parse_int(pid),
            "title"     : title,
            "date"      : date,
            "categories": [cat1, cat2],
            "tags"      : text.split_html(tags),
            "user"      : self.user,
            "body"      : body,
        }

    def _images(self, post):
        imgs = []
        body = post.pop("body")

        for num, img in enumerate(text.extract_iter(body, "<img ", ">"), 1):
            src = text.extract(img, 'src="', '"')[0]
            alt = text.extract(img, 'alt="', '"')[0]

            if src.startswith(self.img_root):
                url = src.replace("-s.", ".")
            else:
                url = text.urljoin(self.root, src)
            name, _, ext = url.rpartition("/")[2].rpartition(".")

            imgs.append({
                "url"      : url,
                "num"      : num,
                "hash"     : name,
                "filename" : alt or name,
                "extension": ext,
                "post"     : post,
            })

        return imgs


class LivedoorBlogExtractor(LivedoorExtractor):
    """Extractor for a user's blog on blog.livedoor.jp"""
    subcategory = "blog"
    pattern = r"(?:https?://)?blog\.livedoor\.jp/(\w+)/?(?:$|[?&#])"
    test = ("http://blog.livedoor.jp/zatsu_ke/", {
        "range": "1-50",
        "count": 50,
        "pattern": r"http://livedoor.blogimg.jp/zatsu_ke/imgs/\w/\w/\w+\.\w+",
        "keyword": {
            "post": {
                "categories": list,
                "date": str,
                "id": int,
                "tags": list,
                "title": str,
                "user": "zatsu_ke"
            },
            "filename": str,
            "hash": r"re:\w{4,}",
            "num": int,
        },
    })

    def posts(self):
        url = "{}/{}".format(self.root, self.user)

        while url:
            page = self.request(url).text
            pos = 0

            while True:
                data, pos = text.extract(page, '.articles.push(', ');', pos)
                if not data:
                    break
                body, pos = text.extract(
                    page,
                    '<div class="article-body-inner">',
                    '<!-- articleBody End -->',
                    pos,
                )
                yield self._load(data, body)

            url = text.extract(page, '<a rel="next" href="', '"', pos)[0]


class LivedoorPostExtractor(LivedoorExtractor):
    """Extractor for images from a blog post on blog.livedoor.jp"""
    subcategory = "post"
    pattern = r"(?:https?://)?blog\.livedoor\.jp/(\w+)/archives/(\d+)"
    test = (
        ("http://blog.livedoor.jp/zatsu_ke/archives/51493859.html", {
            "url": "8826fe623f19dc868e7538e8519bf8491e92a0a2",
            "keyword": "52fcba9253a000c339bcd658572d252e282626af",
        }),
        ("http://blog.livedoor.jp/amaumauma/archives/7835811.html", {
            "url": "fc1d6a9557245b5a27d3a10bf0fa9922ef377215",
            "keyword": "0229072abb5cd8a221df72e0ffdfc13336c0e9ce",
        }),
    )

    def __init__(self, match):
        LivedoorExtractor.__init__(self, match)
        self.post_id = match.group(2)

    def posts(self):
        url = "{}/{}/archives/{}.html".format(
            self.root, self.user, self.post_id)
        page = self.request(url).text

        data, pos = text.extract(page, 'articles :', '</script>')
        body, pos = text.extract(
            page,
            '<div class="article-body-inner">',
            '<!-- articleBody End -->',
            pos,
        )
        return (self._load(data, body),)



# -*- coding: utf-8 -*-

# Copyright 2018-2019 Mike Fährmann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extractors for https://www.newgrounds.com/"""

from .common import Extractor, Message
from .. import text
import json


class NewgroundsExtractor(Extractor):
    """Base class for newgrounds extractors"""
    category = "newgrounds"
    directory_fmt = ("{category}", "{user}")
    filename_fmt = "{category}_{index}_{title}.{extension}"
    archive_fmt = "{index}"

    def __init__(self, match):
        Extractor.__init__(self, match)
        self.user = match.group(1)
        self.root = "https://{}.newgrounds.com".format(self.user)

    def items(self):
        data = self.get_metadata()
        yield Message.Version, 1
        yield Message.Directory, data

        for page_url in self.get_page_urls():
            url, image = self.parse_page_data(page_url)
            image.update(data)
            yield Message.Url, url, text.nameext_from_url(url, image)

    def get_metadata(self):
        """Collect metadata for extractor-job"""
        return {"user": self.user}

    def get_page_urls(self):
        """Return urls of all relevant image pages"""

    def parse_page_data(self, page_url):
        """Collect url and metadata from an image page"""
        page = self.request(page_url).text

        full, pos = text.extract(page, '"full_image_text":', '});')
        desc, pos = text.extract(page, '"og:description" content="', '"', pos)
        rate, pos = text.extract(page, 'class="rated-', '"', pos)
        tags, pos = text.extract(page, '<dd class="tags momag">', '</dd>', pos)

        full = json.loads(full)
        url   , pos = text.extract(full, 'src="', '"')
        title , pos = text.extract(full, 'alt="', '"', pos)
        width , pos = text.extract(full, 'width="', '"', pos)
        height, pos = text.extract(full, 'height="', '"', pos)

        tags = text.split_html(tags)
        tags.sort()

        return url, {
            "title": text.unescape(title),
            "description": text.unescape(desc),
            "width": text.parse_int(width),
            "height": text.parse_int(height),
            "index": text.parse_int(url.rpartition("/")[2].partition("_")[0]),
            "tags": tags,
            "rating": rate,
        }

    def _pagination(self, url):
        headers = {
            "Referer": self.root,
            "X-Requested-With": "XMLHttpRequest",
            "Accept": "application/json, text/javascript, */*; q=0.01",
        }

        while True:
            data = self.request(url, headers=headers).json()

            for year in data["sequence"]:
                for item in data["years"][str(year)]["items"]:
                    page_url = text.extract(item, 'href="', '"')[0]
                    yield text.urljoin(self.root, page_url)

            if not data["more"]:
                return
            url = text.urljoin(self.root, data["more"])


class NewgroundsUserExtractor(NewgroundsExtractor):
    """Extractor for all images of a newgrounds user"""
    subcategory = "user"
    pattern = r"(?:https?://)?([^.]+)\.newgrounds\.com(?:/art)?/?$"
    test = (
        ("https://blitzwuff.newgrounds.com/art", {
            "url": "24b19c4a135a09889fac7b46a74e427e4308d02b",
            "keyword": "f221ddb00f51aa4f17a73809fd9be3c3352fc6b7",
        }),
        ("https://blitzwuff.newgrounds.com/"),
    )

    def get_page_urls(self):
        return self._pagination(self.root + "/art/page/1")


class NewgroundsImageExtractor(NewgroundsExtractor):
    """Extractor for a single image from newgrounds.com"""
    subcategory = "image"
    pattern = (r"(?:https?://)?(?:"
               r"(?:www\.)?newgrounds\.com/art/view/([^/?&#]+)/[^/?&#]+"
               r"|art\.ngfiles\.com/images/\d+/\d+_([^_]+)_([^.]+))")
    test = (
        ("https://www.newgrounds.com/art/view/blitzwuff/ffx", {
            "url": "e7778c4597a2fb74b46e5f04bb7fa1d80ca02818",
            "keyword": "731bf24b9fa7da3bdf094a0f6d181123aae16394",
            "content": "cb067d6593598710292cdd340d350d14a26fe075",
        }),
        ("https://art.ngfiles.com/images/587000/587551_blitzwuff_ffx.png", {
            "url": "e7778c4597a2fb74b46e5f04bb7fa1d80ca02818",
            "keyword": "731bf24b9fa7da3bdf094a0f6d181123aae16394",
        }),
    )

    def __init__(self, match):
        NewgroundsExtractor.__init__(self, match)
        if match.group(2):
            self.user = match.group(2)
            self.page_url = "https://www.newgrounds.com/art/view/{}/{}".format(
                self.user, match.group(3))
        else:
            self.page_url = match.group(0)

    def get_page_urls(self):
        return (self.page_url,)


class NewgroundsVideoExtractor(NewgroundsExtractor):
    """Extractor for all videos of a newgrounds user"""
    subcategory = "video"
    filename_fmt = "{category}_{index}.{extension}"
    pattern = r"(?:https?://)?([^.]+)\.newgrounds\.com/movies/?$"
    test = ("https://twistedgrim.newgrounds.com/movies", {
        "pattern": r"ytdl:https?://www\.newgrounds\.com/portal/view/\d+",
        "count": ">= 29",
    })

    def get_page_urls(self):
        return self._pagination(self.root + "/movies/page/1")

    def parse_page_data(self, page_url):
        return "ytdl:" + page_url, {
            "index": text.parse_int(page_url.rpartition("/")[2])
        }



# -*- coding: utf-8 -*-

# Copyright 2015-2019 Mike Fährmann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extract images from https://hitomi.la/"""

from .common import GalleryExtractor
from .. import text, util
import string


class HitomiGalleryExtractor(GalleryExtractor):
    """Extractor for image galleries from hitomi.la"""
    category = "hitomi"
    pattern = r"(?:https?://)?hitomi\.la/(?:galleries|reader)/(\d+)"
    test = (
        ("https://hitomi.la/galleries/867789.html", {
            "url": "cb759868d090fe0e2655c3e29ebf146054322b6d",
            "keyword": "07536afc5696cb4983a4831ab4c70c1d155f875c",
        }),
        ("https://hitomi.la/galleries/1036181.html", {
            # "aa" subdomain for gallery-id ending in 1 (#142)
            "pattern": r"https://aa\.hitomi\.la/",
        }),
        ("https://hitomi.la/reader/867789.html"),
    )

    def __init__(self, match):
        self.gallery_id = text.parse_int(match.group(1))
        url = "https://hitomi.la/galleries/{}.html".format(self.gallery_id)
        GalleryExtractor.__init__(self, match, url)

    def metadata(self, page):
        pos = page.index('<h1><a href="/reader/')
        extr = text.extract
        title , pos = extr(page, '.html">', '<', pos)
        artist, pos = extr(page, '<h2>', '</h2>', pos)
        group , pos = extr(page, '<td>Group</td><td>', '</td>', pos)
        gtype , pos = extr(page, '<td>Type</td><td>', '</td>', pos)
        lang  , pos = extr(page, '<td>Language</td><td>', '</td>', pos)
        series, pos = extr(page, '<td>Series</td><td>', '</td>', pos)
        chars , pos = extr(page, '<td>Characters</td><td>', '</td>', pos)
        tags  , pos = extr(page, '<td>Tags</td><td>', '</td>', pos)
        date  , pos = extr(page, '<span class="date">', '</span>', pos)
        lang = None if lang == "N/A" else text.remove_html(lang)

        return {
            "gallery_id": self.gallery_id,
            "title"     : text.unescape(title.strip()),
            "artist"    : self._prepare(artist),
            "group"     : self._prepare(group),
            "parody"    : self._prepare(series),
            "characters": self._prepare(chars),
            "tags"      : self._prepare(tags),
            "type"      : text.remove_html(gtype).capitalize(),
            "lang"      : util.language_to_code(lang),
            "language"  : lang,
            "date"      : date,
        }

    def images(self, page):
        # see https://ltn.hitomi.la/common.js
        offset = self.gallery_id % 2 if self.gallery_id % 10 != 1 else 0
        subdomain = chr(97 + offset) + "a"
        base = "https://" + subdomain + ".hitomi.la/galleries/"

        return [
            (base + urlpart, None)
            for urlpart in text.extract_iter(
                page, "'//tn.hitomi.la/smalltn/", ".jpg',"
            )
        ]

    @staticmethod
    def _prepare(value):
        return [
            text.unescape(string.capwords(v))
            for v in text.extract_iter(value or "", '.html">', '<')
        ]
